using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Moq;
using Moq.Protected;
using Newtonsoft.Json;
using HttpClientExample.Controllers;
using Microsoft.AspNetCore.Mvc;
using Xunit;

namespace HttpClientExample.Tests
{
    public class ValuesControllerTest
    {
        [Fact]
        public async Task GetTest()
        {
            //Arrange
            string numberJson= JsonConvert.SerializeObject(new List<int>() { 1, 2, 3, 4, 5 });

            var httpMessageHandler = new Mock<HttpMessageHandler>();
            httpMessageHandler.Protected()
                .Setup<Task<HttpResponseMessage>>("SendAsync", ItExpr.IsAny<HttpRequestMessage>(),
                    ItExpr.IsAny<CancellationToken>())
                .Returns(Task.FromResult(new HttpResponseMessage
                {
                    StatusCode = HttpStatusCode.OK,
                    Content = new StringContent(numberJson, Encoding.UTF8, "application/json"),
                }));

            HttpClient httpClient = new HttpClient(httpMessageHandler.Object);
            httpClient.BaseAddress = new Uri(@"http://localhost:63781/v1/");
            httpClient.DefaultRequestHeaders.Accept.Clear();
            httpClient.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

            ValuesController controller = new ValuesController(httpClient);

            //Act
            IActionResult result = await controller.Get();

            //Assert
            OkObjectResult resultObject = result as OkObjectResult;
            Assert.NotNull(resultObject);

            List<int> numbers = resultObject.Value as List<int>;
            Assert.Equal(5, numbers.Count);
        }
    }
}
